@@ -43,12 +43,14 @@
 #include "MagickCore/studio.h"
 #include "MagickCore/exception.h"
 #include "MagickCore/exception-private.h"
+#include "MagickCore/image-private.h"
 #include "MagickCore/locale_.h"
 #include "MagickCore/log.h"
 #include "MagickCore/memory_.h"
 #include "MagickCore/memory-private.h"
 #include "MagickCore/nt-base-private.h"
 #include "MagickCore/registry.h"
+#include "MagickCore/resource_.h"
 #include "MagickCore/string-private.h"
 #include "MagickCore/timer.h"
 #include "MagickCore/timer-private.h"
@@ -340,7 +342,7 @@ MagickExport double GetElapsedTime(TimerInfo *time_info)
 %
 %  GetMagickTime() returns the time as the number of seconds since the Epoch.
 %
-%  The format of the GetElapsedTime method is:
+%  The format of the GetMagickTime method is:
 %
 %      time_t GetElapsedTime(void)
 %
@@ -348,7 +350,7 @@ MagickExport double GetElapsedTime(TimerInfo *time_info)
 MagickExport time_t GetMagickTime(void)
 {
   static time_t
-    constant_magick_time = 0;
+    constant_magick_time = (time_t) 0;
 
   static MagickBooleanType
     epoch_initialized = MagickFalse;
@@ -358,6 +360,7 @@ MagickExport time_t GetMagickTime(void)
       const char
         *source_date_epoch;
 
+      epoch_initialized=MagickTrue;
       source_date_epoch=getenv("SOURCE_DATE_EPOCH");
       if (source_date_epoch != (const char *) NULL)
         {
@@ -368,7 +371,6 @@ MagickExport time_t GetMagickTime(void)
           if ((epoch > 0) && (epoch <= time((time_t *) NULL)))
             constant_magick_time=epoch;
         }
-      epoch_initialized=MagickTrue;
     }
   if (constant_magick_time != 0)
     return(constant_magick_time);
@@ -380,6 +382,41 @@ MagickExport time_t GetMagickTime(void)
 %                                                                             %
 %                                                                             %
 %                                                                             %
++   G e t M a g i c k T T L                                                   %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetMagickTTL() returns the time as the number of seconds to live.
+%
+%  The format of the GetMagickTTL method is:
+%
+%      MagickOffsetType GetMagickTTL(void)
+%
+*/
+MagickPrivate MagickOffsetType GetMagickTTL(void)
+{
+  static time_t
+    magick_epoch = (time_t) 0;
+
+  static MagickBooleanType
+    epoch_initialized = MagickFalse;
+
+  if (epoch_initialized == MagickFalse)
+    {
+      epoch_initialized=MagickTrue;
+      magick_epoch=time((time_t *) NULL);
+    }
+  return((MagickOffsetType) GetMagickResourceLimit(TimeResource)-
+    (GetMagickTime()-magick_epoch));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 +   G e t T i m e r I n f o                                                   %
 %                                                                             %
 %                                                                             %
