@@ -548,13 +548,19 @@ static Image *ReadEmbedImage(const ImageInfo *image_info,Image *image,
   length=(size_t) ((MagickOffsetType) GetBlobSize(image)-TellBlob(image));
   pixel_info=AcquireVirtualMemory(length,sizeof(*pixels));
   if (pixel_info == (MemoryInfo *) NULL)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+    {
+      (void) ThrowMagickException(exception,GetMagickModule(),
+        ResourceLimitError,"MemoryAllocationFailed","`%s'",image->filename);
+      return((Image *) NULL);
+    }
   pixels=(unsigned char *) GetVirtualMemoryBlob(pixel_info);
   stream=ReadBlobStream(image,length,pixels,&count);
   if (count != (ssize_t) length)
     {
       pixel_info=RelinquishVirtualMemory(pixel_info);
-      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+      (void) ThrowMagickException(exception,GetMagickModule(),CorruptImageError,
+        "ImproperImageHeader","`%s'",image->filename);
+      return((Image *) NULL);
     }
   embed_info=AcquireImageInfo();
   (void) FormatLocaleString(embed_info->filename,MagickPathExtent,
@@ -562,7 +568,6 @@ static Image *ReadEmbedImage(const ImageInfo *image_info,Image *image,
   embed_image=BlobToImage(embed_info,stream,(size_t) count,exception);
   embed_info=DestroyImageInfo(embed_info);
   pixel_info=RelinquishVirtualMemory(pixel_info);
-  (void) CloseBlob(image);
   if (embed_image != (Image *) NULL)
     {
       (void) CopyMagickString(embed_image->filename,image->filename,
@@ -695,6 +700,9 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
     if (bmp_info.size > 124)
       ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    if ((bmp_info.file_size != 0) &&
+        ((MagickSizeType) bmp_info.file_size > GetBlobSize(image)))
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     if (bmp_info.offset_bits < bmp_info.size)
       ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     bmp_info.offset_bits=MagickMax(14+bmp_info.size,bmp_info.offset_bits);
@@ -730,8 +738,19 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Microsoft Windows BMP image file.
         */
-        if (bmp_info.size < 40)
-          ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+        switch (bmp_info.size)
+        {
+          case 40: case 52: case 56: case 64: case 78: case 108: case 124:
+          {
+            break;
+          }
+          default:
+          {
+            if (bmp_info.size < 64)
+              ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+            break;
+          }
+        }
         bmp_info.width=(ssize_t) ReadBlobLSBSignedLong(image);
         bmp_info.height=(ssize_t) ReadBlobLSBSignedLong(image);
         bmp_info.planes=ReadBlobLSBShort(image);
@@ -743,9 +762,6 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
             bmp_info.x_pixels=ReadBlobLSBLong(image);
             bmp_info.y_pixels=ReadBlobLSBLong(image);
             bmp_info.number_colors=ReadBlobLSBLong(image);
-            if ((MagickSizeType) bmp_info.number_colors > blob_size)
-              ThrowReaderException(CorruptImageError,
-                "InsufficientImageDataInFile");
             bmp_info.colors_important=ReadBlobLSBLong(image);
           }
         if (image->debug != MagickFalse)
@@ -961,18 +977,26 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
     if (bmp_info.planes != 1)
       ThrowReaderException(CorruptImageError,"StaticPlanesValueNotEqualToOne");
-    if ((bmp_info.bits_per_pixel != 1) && (bmp_info.bits_per_pixel != 4) &&
-        (bmp_info.bits_per_pixel != 8) && (bmp_info.bits_per_pixel != 16) &&
-        (bmp_info.bits_per_pixel != 24) && (bmp_info.bits_per_pixel != 32))
-      ThrowReaderException(CorruptImageError,"UnsupportedBitsPerPixel");
-    if (bmp_info.bits_per_pixel < 16 &&
-        bmp_info.number_colors > (1U << bmp_info.bits_per_pixel))
+    switch (bmp_info.bits_per_pixel)
+    {
+      case 1: case 4: case 8: case 16: case 24: case 32: case 64:
+      {
+        break;
+      }
+      default:
+        ThrowReaderException(CorruptImageError,"UnsupportedBitsPerPixel");
+    }
+    if ((bmp_info.bits_per_pixel < 16) &&
+        (bmp_info.number_colors > (1U << bmp_info.bits_per_pixel)))
       ThrowReaderException(CorruptImageError,"UnrecognizedNumberOfColors");
+    if ((MagickSizeType) bmp_info.number_colors > blob_size)
+      ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
     if ((bmp_info.compression == BI_RLE8) && (bmp_info.bits_per_pixel != 8))
       ThrowReaderException(CorruptImageError,"UnsupportedBitsPerPixel");
     if ((bmp_info.compression == BI_RLE4) && (bmp_info.bits_per_pixel != 4))
       ThrowReaderException(CorruptImageError,"UnsupportedBitsPerPixel");
-    if ((bmp_info.compression == BI_BITFIELDS) && (bmp_info.bits_per_pixel < 16))
+    if ((bmp_info.compression == BI_BITFIELDS) &&
+        (bmp_info.bits_per_pixel < 16))
       ThrowReaderException(CorruptImageError,"UnsupportedBitsPerPixel");
     switch (bmp_info.compression)
     {
@@ -1535,6 +1559,43 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
         break;
       }
+      case 64:
+      {
+        /*
+          Convert DirectColor scanline.
+        */
+        bytes_per_line=4*((image->columns*64+31)/32);
+        for (y=(ssize_t) image->rows-1; y >= 0; y--)
+        {
+          unsigned short
+            *p16;
+
+          p16=(unsigned short *) pixels+((ssize_t) image->rows-y-1)*
+            (ssize_t) bytes_per_line;
+          q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
+          if (q == (Quantum *) NULL)
+            break;
+          for (x=0; x < (ssize_t) image->columns; x++)
+          {
+            SetPixelBlue(image,ScaleShortToQuantum(*p16++),q);
+            SetPixelGreen(image,ScaleShortToQuantum(*p16++),q);
+            SetPixelRed(image,ScaleShortToQuantum(*p16++),q);
+            SetPixelAlpha(image,ScaleShortToQuantum(*p16++),q);
+            q+=GetPixelChannels(image);
+          }
+          if (SyncAuthenticPixels(image,exception) == MagickFalse)
+            break;
+          offset=((MagickOffsetType) image->rows-y-1);
+          if (image->previous == (Image *) NULL)
+            {
+              status=SetImageProgress(image,LoadImageTag,((MagickOffsetType)
+                image->rows-y),image->rows);
+              if (status == MagickFalse)
+                break;
+            }
+        }
+        break;
+      }
       default:
       {
         pixel_info=RelinquishVirtualMemory(pixel_info);
@@ -2265,7 +2326,7 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image,
           for (x=0; x < (ssize_t) image->columns; x++)
           {
             Quantum alpha=GetPixelAlpha(image,p);
-            if (type == 3 && alpha == TransparentAlpha)
+            if ((type == 3) && alpha == TransparentAlpha)
               {
                 *q++=255;
                 *q++=255;
